/*
 * Copyright (C) 2009-2015 Realtek Semiconductor Corp.
 * All Rights Reserved.
 *
 * This program is the proprietary software of Realtek Semiconductor
 * Corporation and/or its licensors, and only be used, duplicated,
 * modified or distributed under the authorized license from Realtek.
 *
 * ANY USE OF THE SOFTWARE OTHER THAN AS AUTHORIZED UNDER
 * THIS LICENSE OR COPYRIGHT LAW IS PROHIBITED.
 *
 * $Revision: 1.1.1.1 $
 * $Date: 2016/08/04 09:29:57 $
 *
 * Purpose : Definition of I2C API
 *
 *
 */

#ifndef __DRV_TC_H__
#define __DRV_TC_H__

/*
 * Include Files
 */


/*
 * Symbol Definition
 */

/*
 * Data Declaration
 */

typedef enum drv_tc_id_e
{
    TC_ID0 = 0,
    TC_ID1,
    TC_ID2,
    TC_ID3,
    TC_ID4,
    TC_ID_MAX,
} drv_tc_id_t;

typedef enum drv_tc_mode_e
{
    TC_MODE_COUNTER = 0,    /* this is defined according to register book */
    TC_MODE_TIMER = 1,      /* this is defined according to register book */
} drv_tc_mode_t;

typedef enum drv_tc_intState_e
{
    TC_INTERRUPT_PENDING = 0,
    TC_INTERRUPT_PENDING_NO,
} drv_tc_intState_t;



typedef struct tc_dataReg_s
{
    uint32          notUsed1:4;
    uint32          tc_data:28;
} tc_dataReg_t;

typedef struct tc_cntReg_s
{
    uint32          notUsed1:4;
    uint32          tc_data:28;
} tc_cntReg_t;

typedef struct tc_ctlReg_s
{
    uint32          notUsed1:3;
    uint32          tcEnable:1;
    uint32          notUsed2:3;
    uint32          tcMode:1;
    uint32          notUsed3:8;
    uint32          tcDivFactor:16;
} tc_ctlReg_t;

typedef struct tc_intReg_s
{
    uint32          notUsed1:11;
    uint32          intEnable:1;
    uint32          notUsed2:3;
    uint32          intPending:1;
    uint32          notUsed3:16;
} tc_intReg_t;


/*
 * Function Declaration
 */


/* Function Name:
 *      drv_tc_init
 * Description:
 *      Initialize TC module.
 * Input:
 *      unit                - unit id
 * Output:
 *      None
 * Return:
 *      RT_ERR_OK
 *      RT_ERR_FAILED
 *      RT_ERR_UNIT_ID      - invalid unit id
 *      RT_ERR_NULL_POINTER - input parameter may be null pointer
 * Applicable:
 *      8390
 * Note:
 *      Must initialize i2c module before calling any i2c APIs.
 */
int32
drv_tc_init(uint32 unit);


int32 drv_tc_enable_set(uint32 unit,drv_tc_id_t id, rtk_enable_t enable);
int32 drv_tcMode_set(uint32 unit, drv_tc_id_t id, drv_tc_mode_t mode);
int32 drv_tcDivFactor_set(uint32 unit, drv_tc_id_t id,uint32 divFactor);
int32 drv_tcDataInitValue_set(uint32 unit, drv_tc_id_t id, uint32 init_value);
int32 drv_tcIntEnable_set(uint32 unit, drv_tc_id_t id, rtk_enable_t enable);
int32 drv_tcIntState_get(uint32 unit, drv_tc_id_t id, drv_tc_intState_t *state);
int32 drv_tcIntState_clear(uint32 unit, drv_tc_id_t id);
int32 drv_tcCounterValue_get(uint32 unit, drv_tc_id_t id,uint32 *value);











#endif /* __DRV_TC_H__ */

